from django.core.mail import EmailMessage


class SMSMessage(EmailMessage):
	"""
	https://docs.djangoproject.com/en/1.8/topics/email/#the-emailmessage-class
	"""
	def __init__(self, body, from_number, to, connection=None):
		bcc = None
		attachments = None
		headers = None
		cc = None
		reply_to = None
		super().__init__(None, body, from_number, to, bcc, connection, attachments, headers, cc, reply_to)

	def get_connection(self, fail_silently=False):
		from smsish.sms import get_sms_connection as get_connection
		if not self.connection:
			self.connection = get_connection(fail_silently=fail_silently)
		return self.connection



from django.core import mail
from django.test import TestCase
from django.test.utils import override_settings
from django.test.utils import captured_stdout

from smsish.sms import send_sms
from smsish.sms.message import SMSMessage

FROM_NUMBER = "+15005550006"
TO_NUMBER = "+15005550006"


@override_settings(SMS_BACKEND='smsish.sms.backends.dummy.SMSBackend')
class SMSMessageTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			FROM_NUMBER,
			[TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", TO_NUMBER, [])

	def test_create_with_subject_not_allowed(self):
		with self.assertRaises(TypeError):
			SMSMessage("Subject", "Body", FROM_NUMBER, [TO_NUMBER])

	def test_recipients(self):
		sms = self.sms
		recipients = sms.recipients()
		self.assertEqual(recipients, [TO_NUMBER])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()

		# Test that one message has been sent.
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", FROM_NUMBER, [TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()

		# Test that no message has been sent.
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)


@override_settings(SMS_BACKEND='smsish.sms.backends.console.SMSBackend')
class SendSMSUsingConsoleTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			FROM_NUMBER,
			[TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", TO_NUMBER, [])

	def test_send(self):
		with captured_stdout() as stdout:
			sms = self.sms
			numSent = sms.send()
			self.assertEqual(numSent, 1)
			output = stdout.getvalue()
			self.assertTrue("Subject: None" in output)
			self.assertTrue("From: +15005550006" in output)
			self.assertTrue("To: +15005550006" in output)
			self.assertTrue("Body" in output)

		with captured_stdout() as stdout:
			sms_no_recipients = self.sms_no_recipients
			numSent = sms_no_recipients.send()
			self.assertEqual(numSent, 0)
			self.assertEqual(stdout.getvalue(), "")

	def test_send_sms(self):
		with captured_stdout() as stdout:
			numSent = send_sms("Body", FROM_NUMBER, [TO_NUMBER])
			self.assertEqual(numSent, 1)
			self.assertEqual(len(mail.outbox), 0)
			output = stdout.getvalue()
			self.assertTrue("Subject: None" in output)
			self.assertTrue("From: +15005550006" in output)
			self.assertTrue("To: +15005550006" in output)
			self.assertTrue("Body" in output)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)


@override_settings(SMS_BACKEND='smsish.sms.backends.twilio.SMSBackend')
class SendSMSUsingTwilioTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			FROM_NUMBER,
			[TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", FROM_NUMBER, [TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)


@override_settings(
	SMS_BACKEND='smsish.sms.backends.filebased.SMSBackend',
	EMAIL_FILE_PATH="outbox",)
class SendSMSUsingFilebasedTestCase(TestCase):
	"""
	TODO: Test that files are actually written: see https://github.com/django/django/blob/master/tests/mail/tests.py#L876.
	"""
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			FROM_NUMBER,
			[TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_sms(self):
		numSent = send_sms("Body", FROM_NUMBER, [TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 0)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)


@override_settings(SMS_BACKEND='smsish.sms.backends.locmem.SMSBackend')
class SendSMSUsingLocmemTestCase(TestCase):
	def setUp(self):
		self.sms = SMSMessage(
			"Body",
			FROM_NUMBER,
			[TO_NUMBER]
		)
		self.sms_no_recipients = SMSMessage("Body", TO_NUMBER, [])

	def test_send(self):
		sms = self.sms
		numSent = sms.send()
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 1)

	def test_send_sms(self):
		numSent = send_sms("Body", FROM_NUMBER, [TO_NUMBER])
		self.assertEqual(numSent, 1)
		self.assertEqual(len(mail.outbox), 1)

	def test_send_to_nobody(self):
		sms = self.sms_no_recipients
		numSent = sms.send()
		self.assertEqual(numSent, 0)
		self.assertEqual(len(mail.outbox), 0)



"""
Tools for sending SMS messages.
"""
from __future__ import unicode_literals

from django.conf import settings
from django.utils.module_loading import import_string

from smsish.sms.message import (
	SMSMessage,
)

__all__ = [
	'get_sms_connection',
	'send_sms'
]


def get_sms_connection(backend=None, fail_silently=False, **kwds):
	"""Load an sms backend and return an instance of it.
	If backend is None (default) settings.SMS_BACKEND is used.
	Both fail_silently and other keyword arguments are used in the
	constructor of the backend.
	"""
	klass = import_string(backend or settings.SMS_BACKEND)
	return klass(fail_silently=fail_silently, **kwds)


def send_sms(message, from_number, recipient_list, fail_silently=False, auth_user=None, auth_password=None, connection=None):
	"""
	Easy wrapper for sending a single message to a recipient list. All members
	of the recipient list will see the other recipients in the 'To' field.
	If auth_user is None, the EMAIL_HOST_USER setting is used.
	If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.
	Note: The API for this method is frozen. New code wanting to extend the
	functionality should use the EmailMessage class directly.
	"""
	connection = connection or get_sms_connection(username=auth_user, password=auth_password, fail_silently=fail_silently)
	mail = SMSMessage(message, from_number, recipient_list, connection=connection)

	return mail.send()



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
	README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
	name='django-smsish',
	version='1.1',
	packages=[
		'smsish',
		'smsish.sms',
	],
	include_package_data=True,
	license='MIT',  # example license
	description='A simple Django app to send SMS messages using an API similar to that of django.core.mail.',
	long_description=README,
	url='https://github.com/RyanBalfanz/django-smsish',
	author='Ryan Balfanz',
	author_email='ryan@ryanbalfanz.net',
	classifiers=[
		'Environment :: Web Environment',
		'Framework :: Django',
		'Framework :: Django :: 1.8',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.5',
		'Topic :: Communications',
		'Topic :: Communications :: Telephony',
	],
)



from django.conf import settings
from django.core.mail.backends.base import BaseEmailBackend

from twilio.rest import TwilioRestClient


class SMSBackend(BaseEmailBackend):
	def __init__(self, fail_silently=False, **kwargs):
		super().__init__(fail_silently=fail_silently)
		self.connection = None

	def open(self):
		"""
		Ensures we have a connection to the SMS gateway. Returns whether or not a new connection was required (True or False).
		"""
		if self.connection:
			# Nothing to do if the connection is already open.
			return False

		self.connection = self._get_twilio_client()
		return True

	def close(self):
		"""Closes the connection to the email server."""
		del self.connection

	def send_messages(self, email_messages):
		"""
		Sends one or more SMSMessage objects and returns the number of text messages sent.
		"""
		if not email_messages:
			return
		new_conn_created = self.open()
		if not self.connection:
			# We failed silently on open().
			# Trying to send would be pointless.
			return
		num_sent = 0
		for message in email_messages:
			sent = self._send(message)
			if sent:
				num_sent += 1
			if new_conn_created:
				self.close()
		return num_sent

	def _send(self, email_message):
		"""A helper method that does the actual sending."""
		if not email_message.recipients():
			return False
		from_email = email_message.from_email
		recipients = email_message.recipients()
		try:
			self.connection.messages.create(
				to=recipients,
				from_=from_email,
				body=email_message.body
			)
		except Exception:
			if not self.fail_silently:
				raise
			return False
		return True

	def _get_twilio_client(self):
		account = settings.TWILIO_ACCOUNT_SID
		token = settings.TWILIO_AUTH_TOKEN
		client = TwilioRestClient(account, token)
		return client
